package s3scj;

import javax.realtime.PeriodicParameters;
import javax.realtime.PriorityParameters;
import javax.realtime.PriorityScheduler;
import javax.realtime.RealtimeThread;
import javax.realtime.RelativeTime;
import javax.safetycritical.AperiodicEventHandler;
import javax.safetycritical.AperiodicReleaseParameters;
import javax.safetycritical.ManagedMemory;
import javax.safetycritical.Mission;
import javax.safetycritical.NoHeapRealtimeThread;
import javax.safetycritical.PeriodicEventHandler;
import javax.safetycritical.S3StorageConfigurationParameters;
import javax.safetycritical.S3Terminal;
import javax.safetycritical.Safelet;
import javax.safetycritical.SingleMissionSequencer;
import javax.safetycritical.Terminal;
import javax.safetycritical.annotate.Level;

public abstract class MainSafelet implements Safelet {

	protected Properties _prop = new Properties();

	protected Thread _launcher;

	public MainSafelet() {
		_prop.parseArgs(S3Terminal.getTerminal().readAll());
	}

	/* ----------------- Methods ------------------- */

	public void setup() {
		_launcher = Thread.currentThread();
		Terminal.getTerminal().writeln(getInfo());
	}

	public void teardown() {
		Terminal.getTerminal().writeln(report());
	}

	public Level getLevel() {
		return _prop._level;
	}

	protected abstract String getInfo();

	protected abstract String report();

	/* -------------- Wrapped Classes -------------- */
	public abstract class GeneralMission extends Mission {
		@Override
		public long missionMemorySize() {
			return _prop._missionMemSize;
		}

		@Override
		protected void cleanup() {
			super.cleanup();
			_launcher.interrupt();
		}
	}

	public class GeneralSingleMissionSequencer extends SingleMissionSequencer {
		public GeneralSingleMissionSequencer(Mission mission) {
			super(new PriorityParameters(_prop._priority), mission);
		}
	}

	public abstract class GeneralPeriodicEventHandler extends
			PeriodicEventHandler {

		public GeneralPeriodicEventHandler() {
			super(new PriorityParameters(_prop._priority),
					new PeriodicParameters(new RelativeTime(_prop._iDelay, 0),
							new RelativeTime(_prop._period, 0)),
					_prop._schedObjMemSize);
		}
	}

	public abstract class GeneralAperiodicEventHandler extends
			AperiodicEventHandler {

		public GeneralAperiodicEventHandler() {
			super(
					new PriorityParameters(_prop._priority),
					new AperiodicReleaseParameters(null, null),
					new S3StorageConfigurationParameters(_prop._schedObjMemSize));
		}

		public GeneralAperiodicEventHandler(String name) {
			super(
					new PriorityParameters(_prop._priority),
					new AperiodicReleaseParameters(null, null),
					new S3StorageConfigurationParameters(_prop._schedObjMemSize),
					name);
		}
	}

	public class GeneralNoHeapRealtimeThread extends NoHeapRealtimeThread {

		public GeneralNoHeapRealtimeThread() {
			super(
					new PriorityParameters(_prop._priority),
					new S3StorageConfigurationParameters(_prop._schedObjMemSize),
					null);
		}
	}
	
	public class Terminator extends PeriodicEventHandler {

		public Terminator() {
			super(new PriorityParameters(PriorityScheduler.instance()
					.getMaxPriority()), new PeriodicParameters(
					new RelativeTime(_prop._duration, 0), new RelativeTime(
							Long.MAX_VALUE, 0)), 0, "Terminator");
		}

		@Override
		public void handleEvent() {
			((ManagedMemory)RealtimeThread.getCurrentMemoryArea()).getManager().getMission().requestSequenceTermination();
		}
	}
}
