package s3scj.tck;

import javax.realtime.RelativeTime;
import javax.safetycritical.MissionSequencer;
import javax.safetycritical.System;
import javax.safetycritical.Terminal;

public class TestMisc extends TestCase {
	private MissionSequencer _sequencer = new GeneralSingleMissionSequencer(
			new GeneralMission() {

				volatile int _ready;
				volatile int _done;

				@Override
				public void initialize() {

					final RelativeTime delay = new RelativeTime(
							_prop._duration, 0);

					final Thread sleep = new GeneralNoHeapRealtimeThread() {
						@Override
						public void run() {
							try {
								_ready++;
								System.sleep(delay);
								fail("Error occurred in javax.safetycritical.System.sleep(RelativeTime)");
							} catch (InterruptedException e) {
								// expected
							}
							_done++;
						}
					};

					final Thread sleepBusy = new GeneralNoHeapRealtimeThread() {
						@Override
						public void run() {
							try {
								_ready++;
								System.sleep(delay, true);
								fail("Error occurred in javax.safetycritical.System.sleep(RelativeTime, boolean true)");
							} catch (InterruptedException e) {
								// expected
							}
							_done++;
						}
					};
					final Thread sleepNonBusy = new GeneralNoHeapRealtimeThread() {
						@Override
						public void run() {
							try {
								_ready++;
								System.sleep(delay, false);
								fail("Error occurred in javax.safetycritical.System.sleep(RelativeTime, boolean false)");
							} catch (InterruptedException e) {
								// expected
							}
							_done++;
						}
					};
					final Thread sleepNonInt = new GeneralNoHeapRealtimeThread() {
						@Override
						public void run() {
							_ready++;
							try {
								System.sleepNonInterruptable(delay);
							} catch (Throwable t) {
								fail("Error occurred in javax.safetycritical.System.sleepNonInterruptable(RelativeTime)");
							}
							_done++;

						}
					};
					final Thread sleepNonIntBusy = new GeneralNoHeapRealtimeThread() {
						@Override
						public void run() {
							_ready++;
							try {
								System.suspendNonInterruptable(delay, true);
							} catch (Throwable t) {
								fail("Error occurred in javax.safetycritical.System.sleepNonInterruptable(RelativeTime, boolean true)");
							}
							_done++;
						}
					};
					final Thread sleepNonIntNonBusy = new GeneralNoHeapRealtimeThread() {
						@Override
						public void run() {
							_ready++;
							try {
								System.suspendNonInterruptable(delay, false);
							} catch (Throwable t) {
								fail("Error occurred in javax.safetycritical.System.sleepNonInterruptable(RelativeTime, boolean false)");
							}
							_done++;
						}
					};

					sleep.start();
					sleepBusy.start();
					sleepNonBusy.start();
					sleepNonInt.start();
					sleepNonIntBusy.start();
					sleepNonIntNonBusy.start();

					new GeneralPeriodicEventHandler() {
						@Override
						public void handleEvent() {
							// Test Terminal
							Terminal
									.getTerminal()
									.write(
											"Testing Unicode print (you should see an "
													+ "accented e and parallel lines): ");
							Terminal.getTerminal().writeln(
									(char) 2405 + " " + (char) 233);

							// Test sleep function family in System

							try {
								long timeout = 1000000;

								while (_ready < 6 && timeout-- > 0)
									Thread.yield();

								if (timeout <= 0)
									fail("Time out in TestMisc");
								if (_done > 0)
									fail("Error occurred in javax.safetycritical.System.sleep family (threads not sleep)");

								sleep.interrupt();
								sleepBusy.interrupt();
								sleepNonBusy.interrupt();
								sleepNonInt.interrupt();
								sleepNonIntBusy.interrupt();
								sleepNonIntNonBusy.interrupt();

								timeout = 1000000;

								while (_done < 6 && timeout-- > 0)
									Thread.yield();

								if (timeout <= 0)
									fail("Time out in TestMisc");
								
							} catch (Throwable t) {
								fail(t.getMessage());
							} finally {
								_sequencer.requestTermination();
								requestSequenceTermination();
							}
						}
					};
				}
			});

	public MissionSequencer getSequencer() {
		return _sequencer;
	}
}
