package s3scj.tck;

import javax.realtime.Clock;
import javax.realtime.PriorityParameters;
import javax.safetycritical.Mission;
import javax.safetycritical.MissionSequencer;

/**
 * @author leizhao
 * 
 * Assertion:
 *  - missions can be executed sequentially
 */
public class TestSchedule400 extends TestCase {

	private final int _nMissions = 3;

	private final Clock _CLOCK = Clock.getRealtimeClock();
	private long[] _timeline = new long[_nMissions * 2];
	private int _counter;

	public MissionSequencer getSequencer() {
		return new MyMissionSequencer();
	}

	class MyMissionSequencer extends MissionSequencer {

		private Mission[] _missions = null;

		private int _counter;

		public MyMissionSequencer() {
			super(new PriorityParameters(_prop._priority));
			int i;
			_counter = 0;
			_missions = new Mission[_nMissions + 1];
			for (i = 0; i < _nMissions; i++) 
				_missions[i] = new MyMission();
			
			_missions[i] = new FinalMission();
		}

		@Override
		protected Mission getInitialMission() {
			return _missions[_counter++];
		}

		@Override
		protected Mission getNextMission() {
			return _counter < _nMissions + 1 ? _missions[_counter++] : null;
		}
	}

	class MyMission extends GeneralMission {

		@Override
		public void initialize() {
			/*
			 * record the start time of the mission
			 */
			_timeline[_counter++] = _CLOCK.getTime().getMilliseconds();

			new GeneralPeriodicEventHandler() {
				@Override
				public void handleEvent() {
					for (int i = 0; i < 50000; i++)
						;
					terminate(false);
				}
			};
		}

		@Override
		protected void cleanup() {
			// record the finish time of the mission
			_timeline[_counter++] = _CLOCK.getTime().getMilliseconds();
		}
	}

	class FinalMission extends GeneralMission {
		@Override
		public void initialize() {

			new GeneralPeriodicEventHandler() {
				@Override
				public void handleEvent() {
					if (_counter != _nMissions * 2)
						fail("Failed to launch all missions");

					for (int i = 0; i < _nMissions * 2 - 1; i++)
						if (_timeline[i] > _timeline[i + 1])
							fail("Mission timelines overlap");
					requestSequenceTermination();
				}
			};
		}
	}
}
