package s3scj.tck;

import javax.realtime.ImmortalMemory;
import javax.realtime.MemoryArea;
import javax.realtime.RealtimeThread;
import javax.realtime.ScopedMemory;
import javax.safetycritical.AperiodicEvent;
import javax.safetycritical.MissionSequencer;

/**
 * @author leizhao
 * 
 * Assertion: - only no-heap and non-daemon RTSJ schedulable objects supported
 */
public class TestSchedule401 extends TestCase {

	private volatile boolean[] _check = new boolean[3];

	private final int _AEHCheckIndex = 0;

	private final int _PEHCheckIndex = 1;

	private final int _NHRTCheckIndex = 2;

	private volatile int _done = 0;

	public MissionSequencer getSequencer() {
		return new GeneralSingleMissionSequencer(new GeneralMission() {

			@Override
			public void initialize() {

				final AperiodicEvent event = new AperiodicEvent(
						new GeneralAperiodicEventHandler() {
							@Override
							public void handleEvent() {
								RealtimeThread rtthread = RealtimeThread
										.currentRealtimeThread();
								if (rtthread.isDaemon())
									fail("AEH should be non-daemon");

								MemoryArea mem = RealtimeThread
										.getCurrentMemoryArea();
								if (!(mem instanceof ImmortalMemory)
										&& !(mem instanceof ScopedMemory))
									fail("AEH should be non-heap");

								_check[_AEHCheckIndex] = true;
								_done++;
								if (_done >= _check.length)
									requestSequenceTermination();
							}
						});

				new GeneralPeriodicEventHandler() {
					@Override
					public void handleEvent() {
						event.fire();
						RealtimeThread rtthread = RealtimeThread
								.currentRealtimeThread();
						if (rtthread.isDaemon())
							fail("PEH should be non-daemon");

						MemoryArea mem = RealtimeThread.getCurrentMemoryArea();
						if (!(mem instanceof ImmortalMemory)
								&& !(mem instanceof ScopedMemory))
							fail("PEH should be non-heap");

						_check[_PEHCheckIndex] = true;
						_done++;
						if (_done >= _check.length)
							requestSequenceTermination();
					}
				};

				new GeneralNoHeapRealtimeThread() {
					@Override
					public void run() {
						RealtimeThread rtthread = RealtimeThread
								.currentRealtimeThread();
						if (rtthread.isDaemon()) {
							fail("NHRT should be non-daemon");
						}

						MemoryArea mem = RealtimeThread.getCurrentMemoryArea();
						if (!(mem instanceof ImmortalMemory)
								&& !(mem instanceof ScopedMemory))
							fail("NHRT should be non-heap");

						_check[_NHRTCheckIndex] = true;
						_done++;
						if (_done >= _check.length)
							requestSequenceTermination();
					}
				}.start();
			}

			@Override
			public void cleanup() {
				if(!_check[_AEHCheckIndex])
					fail("AEH not executed");
				if(!_check[_PEHCheckIndex])
					fail("PEH not executed");
				if(!_check[_NHRTCheckIndex])
					fail("NHRT not executed");
				super.cleanup();
			}
		});
	}
}
