package s3scj.tck;

import javax.realtime.PeriodicParameters;
import javax.realtime.PriorityParameters;
import javax.realtime.PriorityScheduler;
import javax.realtime.RelativeTime;
import javax.safetycritical.AperiodicEvent;
import javax.safetycritical.MissionSequencer;
import javax.safetycritical.PeriodicEventHandler;
import javax.safetycritical.System;

/**
 * @author leizhao
 * 
 *         Assertion:
 * 
 *         - The default ceiling for locks is
 *         PriorityScheduler.instance().getMaxPriority()
 * 
 *         - Nested calls from one synchronized method to another are allowed
 * 
 *         - The only scheduler is the default RTSJ preemptive priority-based
 *         scheduler with at least 28 priorities; (although if portability is a
 *         main concern, no more than 28 priorities should be used). There is no
 *         support for changing base priorities.
 */
public class TestSchedule402 extends TestCase {

	public MissionSequencer getSequencer() {
		return new GeneralSingleMissionSequencer(new GeneralMission() {

			private long _counter;

			private long _AEHCounter;

			private long _PEHCounter;

			private synchronized void func1() {
				func2();
			}

			private synchronized void func2() {
				func3();
			}

			private synchronized void func3() {
				_counter++;
			}

			@Override
			public void initialize() {
				/*
				 * Test release parameters: PeriodicParameters and
				 * AperiodicReleaseParameters, of which the features include
				 * start, period, deadline, missingHandler
				 * 
				 * FIXME: the semantics of the two parameters are still in
				 * discussion, such as support to deadline and missing handler.
				 * 
				 * Should adjust the tests to comply with the final semantics
				 * once they are decided.
				 */
				final AperiodicEvent event = new AperiodicEvent(
						new GeneralAperiodicEventHandler[] { 
								new GeneralAperiodicEventHandler(
								"AEHCounter") {
							@Override
							public void handleEvent() {
								_AEHCounter++;
							}
						} });

				new PeriodicEventHandler(
						new PriorityParameters(_prop._priority),
						new PeriodicParameters(null, new RelativeTime(200, 0)),
						_prop._schedObjMemSize) {
					@Override
					public void handleEvent() {
						event.fire();
						_PEHCounter++;
					}
				};

				/*
				 * Test nested synchronized function calls. Several threads
				 * compete for the mission object, the func3 should be
				 * eventually touched
				 */
				for (int i = 0; i < 2; i++)
					new GeneralPeriodicEventHandler() {
						@Override
						public void handleEvent() {
							func1();
						}
					};

				new GeneralPeriodicEventHandler() {
					@Override
					public void handleEvent() {
						if (System.getDefaultCeiling() != PriorityScheduler
								.instance().getMaxPriority())
							fail("Default ceiling should equal to the max priority");
						if (PriorityScheduler.instance().getMaxPriority()
								- PriorityScheduler.instance().getMinPriority() < 28)
							fail("Scheduler should provide at least 28 priorities");

						Object obj = new Object();
						System.setCeiling(20, obj);
						// TODO: No way to get the ceiling of obj??
					}
				};

				new Terminator();
			}

			@Override
			protected void cleanup() {
				if (_AEHCounter != _PEHCounter)
					fail("Error occurred in AEH or PEH");
				if (_counter == 0)
					fail("Error occrred in nested synchronization");
				super.cleanup();
			}
		});
	}
}
