package s3scj.tck;

import javax.realtime.Clock;
import javax.realtime.PeriodicParameters;
import javax.realtime.PriorityParameters;
import javax.realtime.PriorityScheduler;
import javax.realtime.RelativeTime;
import javax.safetycritical.MissionSequencer;
import javax.safetycritical.PeriodicEventHandler;

/**
 * @author leizhao
 * 
 *         Assertion:
 * 
 *         - Priority Ceiling Emulation supported one level 1 and 2
 */
public class TestSchedule404 extends TestCase {

	public MissionSequencer getSequencer() {
		return new GeneralSingleMissionSequencer(new GeneralMission() {
			/**
			 * contains the time of lowStart, lowEnd, highStart, highEnd,
			 * medianStart, medianEnd in order
			 */
			private long[] _timeRecord = new long[6];

			private final Clock _CLOCK = Clock.getRealtimeClock();

			@Override
			public void initialize() {
				final MyLock lock = new MyLock();
				final MyFreeLock freelock = new MyFreeLock();

				int MAX_PRIORITY = PriorityScheduler.instance()
						.getMaxPriority();
				int NOR_PRIORITY = PriorityScheduler.instance()
						.getNormPriority();
				int MIN_PRIORITY = PriorityScheduler.instance()
						.getMinPriority();

				PriorityParameters high = new PriorityParameters(
						(MAX_PRIORITY - NOR_PRIORITY) / 2 + NOR_PRIORITY);
				PriorityParameters median = new PriorityParameters(NOR_PRIORITY);
				PriorityParameters low = new PriorityParameters(NOR_PRIORITY
						- (NOR_PRIORITY - MIN_PRIORITY) / 2);

				/*
				 * start time order: low -> high -> median
				 * 
				 * "high" will be blocked on "lock"; but "median" never.If the
				 * priority ceiling protocol is implemented correctly, the
				 * priorities of "median" and "high" should not be inverted,
				 * i.e., they should finish in the order that they start
				 */
				new PeriodicEventHandler(low, new PeriodicParameters(null,
						new RelativeTime(Long.MAX_VALUE, 0)),
						_prop._schedObjMemSize) {
					@Override
					public void handleEvent() {
						lock.doLow();
					}
				};
				new PeriodicEventHandler(high, new PeriodicParameters(
						new RelativeTime(5, 0), new RelativeTime(
								Long.MAX_VALUE, 0)), _prop._schedObjMemSize) {
					@Override
					public void handleEvent() {
						lock.doHigh();
					}
				};
				new PeriodicEventHandler(median, new PeriodicParameters(
						new RelativeTime(10, 0), new RelativeTime(
								Long.MAX_VALUE, 0)), _prop._schedObjMemSize) {
					@Override
					public void handleEvent() {
						freelock.doMedian();
					}
				};

				new Terminator();
			}

			// take around ? ms
			private void killingTime() {
				for (int i = 0; i < 100000000; i++)
					;
			}

			public void cleanup() {
				for (int i = 0; i < 5; i++)
					if (_timeRecord[i] > _timeRecord[i + 1])
						fail("Error in priority ceiling emulation");
				super.cleanup();
			}

			class MyLock {
				public synchronized void doLow() {
					_timeRecord[0] = _CLOCK.getTime().getMilliseconds();
					killingTime();
					_timeRecord[1] = _CLOCK.getTime().getMilliseconds();
				}

				public synchronized void doHigh() {
					_timeRecord[2] = _CLOCK.getTime().getMilliseconds();
					killingTime();
					_timeRecord[3] = _CLOCK.getTime().getMilliseconds();
				}
			}

			class MyFreeLock {
				public synchronized void doMedian() {
					_timeRecord[4] = _CLOCK.getTime().getMilliseconds();
					killingTime();
					_timeRecord[5] = _CLOCK.getTime().getMilliseconds();
				}
			}
		});
	}
}
