package s3scj.tck;

import javax.realtime.Clock;
import javax.realtime.PeriodicParameters;
import javax.realtime.PriorityParameters;
import javax.realtime.PriorityScheduler;
import javax.realtime.RelativeTime;
import javax.safetycritical.MissionSequencer;
import javax.safetycritical.PeriodicEventHandler;

/**
 * @author leizhao
 * 
 *         Assertion:
 * 
 *         - On Level 1 and 2 full preemptively scheduling shall be supported
 */
public class TestSchedule405 extends TestCase {

	public MissionSequencer getSequencer() {
		return new GeneralSingleMissionSequencer(new GeneralMission() {
			/**
			 * contains the time of lowStart, highStart, highEnd, medianStart,
			 * medianEnd, lowEnd in order
			 */
			private long[] _timeRecord = new long[6];

			private final Clock _CLOCK = Clock.getRealtimeClock();

			@Override
			public void initialize() {
				int MAX_PRIORITY = PriorityScheduler.instance()
						.getMaxPriority();
				int NOR_PRIORITY = PriorityScheduler.instance()
						.getNormPriority();
				int MIN_PRIORITY = PriorityScheduler.instance()
						.getMinPriority();

				PriorityParameters high = new PriorityParameters(
						(MAX_PRIORITY - NOR_PRIORITY) / 2 + NOR_PRIORITY);
				PriorityParameters median = new PriorityParameters(NOR_PRIORITY);
				PriorityParameters low = new PriorityParameters(NOR_PRIORITY
						- (NOR_PRIORITY - MIN_PRIORITY) / 2);

				/*
				 * start time order: low -> high -> median
				 * 
				 * high and median are supposed to preempt the execution of low,
				 * so we should have following order:
				 * 
				 * lowStart -> highStart -> highEnd -> medianStart -> medianEnd
				 * -> lowEnd
				 */
				new PeriodicEventHandler(low, new PeriodicParameters(null,
						new RelativeTime(Long.MAX_VALUE, 0)),
						_prop._schedObjMemSize) {
					@Override
					public void handleEvent() {
						// Terminal.getTerminal().writeln("low starts");
						_timeRecord[0] = _CLOCK.getTime().getMilliseconds();
						killingTime();
						_timeRecord[5] = _CLOCK.getTime().getMilliseconds();
						// Terminal.getTerminal().writeln("low finishes");
					}
				};
				new PeriodicEventHandler(high, new PeriodicParameters(
						new RelativeTime(5, 0), new RelativeTime(
								Long.MAX_VALUE, 0)), _prop._schedObjMemSize) {
					@Override
					public void handleEvent() {
						// Terminal.getTerminal().writeln("high starts");
						_timeRecord[1] = _CLOCK.getTime().getMilliseconds();
						killingTime();
						_timeRecord[2] = _CLOCK.getTime().getMilliseconds();
						// Terminal.getTerminal().writeln("high finishes");
					}
				};
				new PeriodicEventHandler(median, new PeriodicParameters(
						new RelativeTime(10, 0), new RelativeTime(
								Long.MAX_VALUE, 0)), _prop._schedObjMemSize) {
					@Override
					public void handleEvent() {
						// Terminal.getTerminal().writeln("median starts");
						_timeRecord[3] = _CLOCK.getTime().getMilliseconds();
						killingTime();
						_timeRecord[4] = _CLOCK.getTime().getMilliseconds();
						// Terminal.getTerminal().writeln("median finishes");
					}
				};

				new Terminator();
			}

			// take around ? ms
			private void killingTime() {
				for (int i = 0; i < 100000000; i++)
					;
			}

			public void cleanup() {
				for (int i = 0; i < 5; i++)
					if (_timeRecord[i] > _timeRecord[i + 1])
						fail("Error in preemptive scheduling");
				super.cleanup();
			}
		});
	}
}
